package com.myapps.kafka.consumer;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.myapps.kafka.email.EmailService;
import com.myapps.kafka.model.dos.DebeziumEvent;
import com.myapps.kafka.model.tres.AuditData;
import com.myapps.kafka.model.tres.AuditEvent;
import com.myapps.kafka.model.uno.Transaccion;
import com.myapps.kafka.producer.KafkaProducerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Service;

import java.util.Arrays;

@Service
@Slf4j
public class KafkaConsumerService {

    private static final Logger log = LoggerFactory.getLogger(KafkaConsumerService.class);

    private static final int MAX_TRANSACCIONES = 10;
    private static final double MONTO_MINIMO_ALERTA = 5000.0;


    private static String[] listaTablas = {"PARAMETROS", "TRANSACCIONES", "USUARIOS"};
    private static String[] listaUsuarios = {"FRAUDE", "GGADMIN123"};
    private static String[] listaOperacionesNoPermitidas = {"DELETE", "UPDATE"};


    @Autowired
    private KafkaProducerService producerService;
    @Autowired
    private EmailService emailService;

    @KafkaListener(topics = "parametros.GGADMIN123.MY_AUDIT_TRAIL")
    public void listen(ConsumerRecord<String, String> record) {
        log.info("Mensaje Entrante");
        log.info("Mensaje recibido -> Key: {}, Value: {}", record.key(), record.value());
        // procesar(record.value());
        //alertar(record.value());
        auditoria(record.value());
    }

    private void auditoria(String value) {
        try {
            // Parseo del evento JSON
            AuditEvent transaccion = mapearDos(value);
            if (transaccion == null || transaccion.getAfter() == null) {
                return;  // Si no hay datos válidos, no procesar
            }

            // Verificamos que el USERID sea "FRAUDE"
            String userId = transaccion.getAfter().getUserId();
            String tabla = transaccion.getAfter().getObjName();
            if (Arrays.asList(listaTablas).contains(tabla) && !Arrays.asList(listaUsuarios).contains(userId)) {
                // Evento que coincide con el usuario "FRAUDE"

                for (String operacion : listaOperacionesNoPermitidas) {
                    String sql = transaccion.getAfter().getSqlText();
                    if (sql != null && sql.toUpperCase().contains(operacion)) {
                        String alertaJson = String.format(
                                "El usuario " + userId + " ejecuto una operacion no permitida en la tabla: " + tabla,
                                userId,
                                transaccion.getAfter().getSessionId(),
                                transaccion.getAfter().getEntryId(),
                                transaccion.getAfter().getnTimestampHash(),
                                transaccion.getAfter().getSqlBind(),
                                transaccion.getAfter().getSqlText()
                        );

                        log.warn("🚨 Alerta generada: {}", alertaJson);
                    } else {
                        log.info("La operación: {} es permitida ", operacion);
                    }
                }

            }

        } catch (Exception e) {
            log.error("Error procesando evento: {}", e.getMessage());
            log.error("Evento problemático: {}", value);
        }

    }

    private void procesar(String value) {
        Transaccion transaccion = mapear(value);
        if (transaccion != null) {
            log.info("Procesando: {}", value);
            int transactionCount = 0;
            double totalAmount = 0.0;

            // Actualizar contadores
            transactionCount++;
            totalAmount += transaccion.getAfter().getMonto();

            // Aplicar reglas de negocio
            boolean alerta = false;
            String motivo = "";

            if (transactionCount >= MAX_TRANSACCIONES) {
                motivo = String.format("Número de transacciones excedido (%d >= %d)",
                        transactionCount, MAX_TRANSACCIONES);
                alerta = true;
                log.info("Se supero el numero de transacciones: {}", motivo);
            }

            if (totalAmount >= MONTO_MINIMO_ALERTA) {
                motivo += String.format("%sMonto acumulado superado (%.2f >= %.2f)",
                        motivo.isEmpty() ? "" : " | ",
                        totalAmount, MONTO_MINIMO_ALERTA);
                alerta = true;
                log.info("Se supero el monto maximo {}", motivo);
            }

            if (alerta) {
                // Construir mensaje de alerta
                String alertaJson = String.format(
                        "{\"usuario_id\":%d,\"total_transacciones\":%d,\"monto_total\":%.2f,\"motivo\":\"%s\"}",
                        transaccion.getAfter().getUsuarioId(), transactionCount, totalAmount, motivo);

                log.info("🚨 Alerta generada: {}", alertaJson);

                producerService.sendMessage(alertaJson);


            }
        }

    }

    private Transaccion mapear(String value) {
        if (value != null) {
            ObjectMapper mapper = new ObjectMapper();
            try {
                return mapper.readValue(value, Transaccion.class);
            } catch (JsonProcessingException e) {
                log.warn("Error al convertir el message:  {}", value);
                return null;
            }
        }
        return null;
    }

    private AuditEvent mapearDos(String value) {
        if (value != null) {
            ObjectMapper mapper = new ObjectMapper();
            try {
                return mapper.readValue(value, AuditEvent.class);
            } catch (JsonProcessingException e) {
                log.warn("Error al convertir el message:  {}", value);
                return null;
            }
        }
        return null;
    }


    public void alertar(String value) {
        String subject = "Alerta: Debezium y Apache Flink";
        String body = value;
        emailService.sendAlertEmail("miguel.claure@myapps.com.bo", subject, body);
    }
/*
    public static void main(String[] args) {
        String json = "{\"before\":null,\"after\":{\"ID\":133265,\"USUARIO_ID\":7,\"MONTO\":77777,\"FECHA\":1743429643452990},\"source\":{\"version\":\"2.1.4.Final\",\"connector\":\"oracle\",\"name\":\"fraude\",\"ts_ms\":1743444043000,\"snapshot\":\"false\",\"db\":\"ORCLCDB\",\"sequence\":null,\"schema\":\"FRAUDE\",\"table\":\"TRANSACCIONES\",\"txId\":\"0a001400b0250200\",\"scn\":\"30883127\",\"commit_scn\":\"30883128\",\"lcr_position\":null,\"rs_id\":null,\"ssn\":0,\"redo_thread\":1,\"user_name\":\"FRAUDE\"},\"op\":\"c\",\"ts_ms\":1743444043795,\"transaction\":null}";
        try {
            ObjectMapper mapper = new ObjectMapper();
            Transaccion transaccion= mapper.readValue(json, Transaccion.class);
            System.out.println(transaccion.getAfter().getUsuarioId());
        } catch (JsonProcessingException e) {
            System.err.println("ERROR: "+ e.getMessage());
        }
    }
    /*
 */
}
